@@ -46,7 +46,7 @@ from pre_workbench.windows.dockwindows import FileBrowserWidget, MdiWindowListWi
 from pre_workbench.windows.dockwindows import RangeTreeDockWidget, RangeListWidget, SelectionHeuristicsConfigWidget, LogWidget
 from pre_workbench.controls.genericwidgets import MemoryUsageWidget, showPreferencesDlg
 from pre_workbench.typeeditor import JsonView
-from pre_workbench.typeregistry import WindowTypes
+from pre_workbench.typeregistry import WindowTypes, DockWidgetTypes
 from pre_workbench.windows.content.hexfile import HexFileWindow
 
 configs.registerOption(SettingsSection('General', 'General', 'Updates', 'Updates'),
@@ -104,13 +104,14 @@ class WorkbenchMain(QMainWindow):
 	def updateChildWindowList(self, obj=None):
 		logging.log(logging.TRACE, "updateChildWindowList")
 		wndList = self.mdiArea.dockWidgetsMap().values()
-		self.dockWidgets["Window List"].updateWindowList(wndList)
+		self.dockWidgets["MdiWindowListWidget"].updateWindowList(wndList)
 
 	def closeEvent(self, e):
 		configs.setValue("MainWindowGeometry", self.saveGeometry())
 		configs.setValue("MainWindowState", self.saveState(123))
 		if not self._checkDirtyChildren():
 			e.ignore()
+			return
 		self._saveChildrenState()
 		self.project.setValue("DockState", self.mdiArea.saveState())
 		super().closeEvent(e)
@@ -119,6 +120,7 @@ class WorkbenchMain(QMainWindow):
 		for wnd in self.mdiArea.dockWidgetsMap().values():
 			if hasattr(wnd.widget(), "maybeSave"):
 				if not wnd.widget().maybeSave():
+					logging.info("close cancelled by %r", wnd.widget())
 					return False
 		return True
 
@@ -331,46 +333,20 @@ class WorkbenchMain(QMainWindow):
 	def _initDockWindows(self):
 		self.dockWidgets = {}
 
-		self.createDockWnd("Project Files", "folder-tree.png", FileBrowserWidget(self.project.projectFolder), ads.LeftDockWidgetArea, showFirstRun=True)
-		self.dockWidgets["Project Files"].on_open.connect(self.openFile)
-
-		self.createDockWnd("Window List", "applications-stack.png", MdiWindowListWidget(), ads.LeftDockWidgetArea)
-
 		self.zoomWindow = DynamicDataWidget()
 		self.zoomWindow.meta_updated.connect(self.onMetaUpdateRaw)
-		self.createDockWnd("Zoom", "document-search-result.png", self.zoomWindow, ads.BottomDockWidgetArea, showFirstRun=True)
+		self.createDockWnd("Zoom", "Zoom", "document-search-result.png", self.zoomWindow, ads.BottomDockWidgetArea, showFirstRun=True)
 		self.zoom_updated.connect(lambda content: self.zoomWindow.setContents(content) if content is not None else None)
 
-		self.createDockWnd("Data Inspector", "user-detective-gray.png", DataInspectorWidget(), ads.BottomDockWidgetArea, showFirstRun=True)
-		self.selected_bytes_updated.connect(self.dockWidgets["Data Inspector"].on_select_bytes)
-
-		self.createDockWnd("Data Source Log", "terminal--exclamation.png", LogWidget("DataSource"), ads.TopDockWidgetArea)
-		self.createDockWnd("Application Log", "bug--exclamation.png", LogWidget(""), ads.TopDockWidgetArea)
-
-		#self.createDockWnd("Grammar Definition Tree", "tree.png", StructInfoTreeWidget())
-		self.createDockWnd("Grammar Definition Code", "tree--pencil.png", StructInfoCodeWidget(), showFirstRun=True)
-
-		self.createDockWnd("Grammar Parse Result", "tree--arrow.png", RangeTreeDockWidget(), showFirstRun=True)
-		self.meta_updated.connect(self.dockWidgets["Grammar Parse Result"].on_meta_update)
-
-		self.createDockWnd("Selected Ranges", "bookmarks.png", RangeListWidget())
-		self.meta_updated.connect(self.dockWidgets["Selected Ranges"].on_meta_update)
-
-		self.createDockWnd("Selection Heuristics", "table-select-cells.png", SelectionHeuristicsConfigWidget())
-
-		#self.createDockWnd("Binwalk", "regular-expression-search-match.png", BinwalkDockWidget(), ads.BottomDockWidgetArea, showFirstRun=False)
-		#self.selected_bytes_updated.connect(self.dockWidgets["Binwalk"].on_select_bytes)
-		self.createDockWnd("Macros", "regular-expression-search-match.png", MacroListDockWidget(),
-						   ads.RightDockWidgetArea, showFirstRun=False)
-
-		self.createDockWnd("External Tools", "toolbox--arrow.png", ExtToolDockWidget(), ads.BottomDockWidgetArea, showFirstRun=False)
-		self.selected_bytes_updated.connect(self.dockWidgets["External Tools"].on_select_bytes)
-
-		self.createDockWnd("Search", "magnifier-left.png", SearchDockWidget(), ads.BottomDockWidgetArea, showFirstRun=False)
-		self.meta_updated.connect(self.dockWidgets["Search"].on_meta_updated)
-
-
+		for typ, meta in DockWidgetTypes.types:
+			widget = typ()
+			self.createDockWnd(meta['name'], meta['title'], meta['icon'], widget, getattr(ads, meta.get('dock', 'Right') + 'DockWidgetArea'), meta.get('showFirstRun', False))
+			if hasattr(widget, 'on_meta_updated'): self.meta_updated.connect(widget.on_meta_updated)
+			if hasattr(widget, 'on_selected_bytes_updated'): self.selected_bytes_updated.connect(widget.on_selected_bytes_updated)
+			if hasattr(widget, 'on_focused_dock_widget_changed'): self.mdiArea.focusedDockWidgetChanged.connect(widget.on_focused_dock_widget_changed)
 
+		self.createDockWnd("Data Source Log", "Data Source Log", "terminal--exclamation.png", LogWidget("DataSource"), ads.TopDockWidgetArea)
+		self.createDockWnd("Application Log", "Application Log", "bug--exclamation.png", LogWidget(""), ads.TopDockWidgetArea)
 
 	def _initUI(self):
 		ads.CDockManager.setConfigFlag(ads.CDockManager.FocusHighlighting, True)
@@ -393,7 +369,6 @@ class WorkbenchMain(QMainWindow):
 		self.setAcceptDrops(True)
 
 		self.mdiArea.focusedDockWidgetChanged.connect(self.onSubWindowActivated)
-		self.mdiArea.focusedDockWidgetChanged.connect(self.dockWidgets["Window List"].onSubWindowActivated)
 
 		# required for actions in "Window" menu
 		self.windowMapper = QSignalMapper(self)
@@ -536,8 +511,12 @@ class WorkbenchMain(QMainWindow):
 		import subprocess
 		self_script = sys.argv[0]
 		# weird Windows magic
-		if not os.path.exists(self_script) and os.path.exists(self_script + '.exe'): self_script += '.exe'
-		cmd_line = [sys.executable, self_script, projectPath]
+		if self_script == sys.executable:  # PyInstaller on Windows
+			cmd_line = [self_script, projectPath]
+		elif not os.path.exists(self_script) and os.path.exists(self_script + '.exe'):  # PIP entry_points on Windows
+			cmd_line = [sys.executable, self_script + '.exe', projectPath]
+		else:  # everything else
+			cmd_line = [sys.executable, self_script, projectPath]
 		logging.info('Starting new instance with cmd line: %r', cmd_line)
 		subprocess.Popen(cmd_line, stdin=subprocess.DEVNULL, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
 
@@ -640,8 +619,8 @@ class WorkbenchMain(QMainWindow):
 			self.mdiArea.addDockWidgetTabToArea(subwnd, self.centralDockWidget.dockAreaWidget())
 		self.updateChildWindowList()
 
-	def createDockWnd(self, name, iconName, widget, area=ads.RightDockWidgetArea, showFirstRun=False):
-		dw=ads.CDockWidget(name)
+	def createDockWnd(self, name, title, iconName, widget, area=ads.RightDockWidgetArea, showFirstRun=False):
+		dw=ads.CDockWidget(title)
 		dw.setObjectName(name)
 		dw.setWidget(widget)
 		dw.setIcon(getIcon(iconName))
