@@ -44,15 +44,18 @@ from pre_workbench.rangetree import RangeTreeWidget
 from pre_workbench.structinfo.parsecontext import AnnotatingParseContext
 from pre_workbench.windows.content.textfile import ScintillaEdit
 from pre_workbench.typeeditor import JsonView
-from pre_workbench.typeregistry import WindowTypes
+from pre_workbench.typeregistry import WindowTypes, DockWidgetTypes
 from pre_workbench.util import PerfTimer, truncate_str
 
 
+
+
+@DockWidgetTypes.register(title="Project Files", icon="folder-tree.png", dock="Left", showFirstRun=True)
 class FileBrowserWidget(QWidget):
-	on_open = pyqtSignal(str)
-	def __init__(self, rootFolder):
+	def __init__(self):
 		super().__init__()
 		self._initUI()
+		rootFolder = pre_workbench.app.CurrentProject.projectFolder
 		self.setRoot(rootFolder)
 
 	def _initUI(self):
@@ -119,7 +122,10 @@ class FileBrowserWidget(QWidget):
 	def saveState(self):
 		if self.tree.currentIndex().isValid():
 			info = self.model.fileInfo(self.tree.currentIndex())
-			return { "sel": info.absoluteFilePath(), "root": self.rootFolder, "hs": self.tree.header().saveState() }
+			sel = info.absoluteFilePath()
+		else:
+			sel = None
+		return { "sel": sel, "root": self.rootFolder, "hs": self.tree.header().saveState() }
 
 	def restoreState(self, state):
 		try:
@@ -133,6 +139,8 @@ class FileBrowserWidget(QWidget):
 		if "hs" in state: self.tree.header().restoreState(state["hs"])
 
 
+
+@DockWidgetTypes.register(title="Window List", icon="applications-stack.png", dock="Left")
 class MdiWindowListWidget(QWidget):
 	def __init__(self):
 		super().__init__()
@@ -173,7 +181,7 @@ class MdiWindowListWidget(QWidget):
 		ctx.addAction("Close all windows", lambda: self.closeAllWindows())
 		ctx.exec(self.list.viewport().mapToGlobal(point))
 
-	def onSubWindowActivated(self, window):
+	def on_focused_dock_widget_changed(self, window):
 		if window is None: return
 		for i in range(self.list.count()):
 			if self.list.item(i).data(QtCore.Qt.UserRole) == window.objectName():
@@ -202,6 +210,7 @@ class StructInfoTreeWidget(QWidget):
 		self.setLayout(windowLayout)
 
 
+@DockWidgetTypes.register(title="Grammar Definition Code", icon="tree--pencil.png", showFirstRun=True)
 class StructInfoCodeWidget(QWidget):
 	def __init__(self):
 		super().__init__()
@@ -226,6 +235,7 @@ class StructInfoCodeWidget(QWidget):
 		self.editor.show()
 
 
+@DockWidgetTypes.register(title="Grammar Parse Result", icon="tree--arrow.png", showFirstRun=True)
 class RangeTreeDockWidget(QWidget):
 	def __init__(self):
 		super().__init__()
@@ -250,7 +260,7 @@ class RangeTreeDockWidget(QWidget):
 		if range is not None and hexView is not None:
 			hexView.selectRange(range, scrollIntoView=True)
 
-	def on_meta_update(self, event_id, param):
+	def on_meta_updated(self, event_id, param):
 		if param is None or not self.isVisible(): return
 		if event_id == "hexview_range":
 			with PerfTimer("RangeTreeWidget update"):
@@ -271,7 +281,7 @@ class RangeTreeDockWidget(QWidget):
 
 
 
-
+@DockWidgetTypes.register(title="Data Inspector", icon="user-detective-gray.png", dock="Bottom", showFirstRun=True)
 class DataInspectorWidget(QWidget):
 	defaultdef = """
 	DataInspector union (ignore_errors=true, endianness=">"){
@@ -306,7 +316,7 @@ class DataInspectorWidget(QWidget):
 	def restoreState(self, data):
 		if "hs" in data: self.fiTreeWidget.header().restoreState(data["hs"])
 
-	def on_select_bytes(self, selbytes):#buffer:ByteBuffer, range:Range):
+	def on_selected_bytes_updated(self, selbytes):#buffer:ByteBuffer, range:Range):
 		if not self.isVisible(): return
 		self.selbytes = selbytes
 		self.parse()
@@ -336,6 +346,7 @@ class DataInspectorWidget(QWidget):
 		self.fiTreeWidget.loadFormatInfo(load_from_file=filespec)
 
 
+@DockWidgetTypes.register(title="Selected Ranges", icon="bookmarks.png")
 class RangeListWidget(QWidget):
 	def __init__(self):
 		super().__init__()
@@ -354,7 +365,7 @@ class RangeListWidget(QWidget):
 		windowLayout.setContentsMargins(0,0,0,0)
 		self.setLayout(windowLayout)
 
-	def on_meta_update(self, event_id, sender):
+	def on_meta_updated(self, event_id, sender):
 		with PerfTimer("RangeListWidget update"):
 			if event_id != "hexview_range" or sender is None or not self.isVisible(): return
 			self.treeView.clear()
@@ -374,6 +385,7 @@ class RangeListWidget(QWidget):
 				# TODO ...on click: self.selectRange(d)
 
 
+@DockWidgetTypes.register(title="Macros", icon="regular-expression-search-match.png", dock="Right", showFirstRun=False)
 class MacroListDockWidget(QWidget):
 	CONTAINER_ROLE = QtCore.Qt.UserRole + 100
 	MACRO_NAME_ROLE = QtCore.Qt.UserRole + 101
@@ -387,8 +399,9 @@ class MacroListDockWidget(QWidget):
 		self.treeView.setColumnCount(1)
 		self.treeView.setColumnWidth(1, 300)
 		self.treeView.headerItem().setText(0, "Name")
-		self.treeView.customContextMenuRequested.connect(self.customContextMenuRequested)
+		self.treeView.customContextMenuRequested.connect(self._customContextMenuRequested)
 		self.treeView.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
+		self.treeView.itemDoubleClicked.connect(self._onDblClick)
 		windowLayout = QVBoxLayout()
 		windowLayout.addWidget(self.treeView)
 		windowLayout.setContentsMargins(0,0,0,0)
@@ -409,7 +422,7 @@ class MacroListDockWidget(QWidget):
 			item.setData(0, MacroListDockWidget.CONTAINER_ROLE, container)
 			item.setData(0, MacroListDockWidget.MACRO_NAME_ROLE, name)
 
-	def customContextMenuRequested(self, point):
+	def _customContextMenuRequested(self, point):
 		item = self.treeView.itemAt(point)
 		ctx = QMenu("Context menu", self)
 		if item:
@@ -437,6 +450,11 @@ class MacroListDockWidget(QWidget):
 			container.deleteMacro(macroname)
 			self._loadList()
 
+	def _onDblClick(self, item: QTreeWidgetItem, columnIdx: int):
+		container = item.data(0, MacroListDockWidget.CONTAINER_ROLE)
+		macroname = item.data(0, MacroListDockWidget.MACRO_NAME_ROLE)
+		self.executeMacro(container.getMacro(macroname))
+
 	def executeMacro(self, macro):
 		if macro.input_type == Macro.TYPE_NONE:
 			macro.execute(None)
@@ -493,6 +511,7 @@ class MacroListDockWidget(QWidget):
 
 
 
+@DockWidgetTypes.register(title="Selection Heuristics", icon="table-select-cells.png")
 class SelectionHeuristicsConfigWidget(QWidget):
 	HELPER_ROLE = QtCore.Qt.UserRole + 100
 
@@ -569,6 +588,7 @@ class RpcDockWidget(QWidget):
 		pass
 
 
+# @DockWidgetTypes.register(title="Binwalk", icon="regular-expression-search-match.png", dock="Bottom", showFirstRun=False)
 class BinwalkDockWidget(QWidget):
 	def __init__(self):
 		super().__init__()
@@ -590,7 +610,7 @@ class BinwalkDockWidget(QWidget):
 		windowLayout.setContentsMargins(0,0,0,0)
 		self.setLayout(windowLayout)
 
-	def on_select_bytes(self, selBytes):
+	def on_selected_bytes_updated(self, selBytes):
 		self.selBytes = selBytes
 
 	def _runBinwalk(self):
@@ -608,6 +628,7 @@ class BinwalkDockWidget(QWidget):
 				x.setText(2, result.description)
 
 
+@DockWidgetTypes.register(title="External Tools", icon="toolbox--arrow.png", dock="Bottom", showFirstRun=False)
 class ExtToolDockWidget(QWidget):
 	def __init__(self):
 		super().__init__()
@@ -633,7 +654,7 @@ class ExtToolDockWidget(QWidget):
 		self.cmdLineEdit.clear()
 		self.cmdLineEdit.addItems(configs.getValue("ExtToolMru", []))
 
-	def on_select_bytes(self, selBytes):
+	def on_selected_bytes_updated(self, selBytes):
 		self.selBytes = selBytes
 
 	def _runTool(self):
@@ -657,6 +678,7 @@ class ExtToolDockWidget(QWidget):
 
 
 
+@DockWidgetTypes.register(title="Search", icon="magnifier-left.png", dock="Bottom", showFirstRun=False)
 class SearchDockWidget(QWidget):
 	def __init__(self):
 		super().__init__()
